/*
  Blank Simple Project.c
  http://learn.parallax.com/propeller-c-tutorials 
*/

#include "simpletools.h"                      // Include simple tools
#include "mstimer.h"                          // Cog Timer Function
#include "fdserial.h"                         // Full Duplex Comm
#include "Mavlink/common/mavlink.h"           // Mavlink decoder/encoder
#include "ws2812.h"


int MavDecode(mavlink_message_t *);
void Reading();
void Circle(void *par);
void FixColor(int);
void request_mavlink_rates(void);


#define Rx 4
#define Tx 5
#define Baud 57600
#define LCNT 16
#define LED_PIN 11

typedef struct  _MavData
{
  int System;
  int Compent;
  int Type;
  int Armed;
  int Custom;
  int Mode;
  int BatteryRemaining;
  int BatteryVoltage;
  int Latitude;
  int Longitude;
  int Altitude;
  int Satelites;
  int HDOP;
  int GPSFix;
  int Heading;
  int RelativeAltitude;
  int Throttle;
  int Pitch;
  int Roll;
  int Yaw;
  int WapPoint;
  }  mavdata_t;

mavlink_system_t mavlink_system;
static int packet_drops = 0;
static int parse_errors = 0;

char SMode[][10] = { "Stabilize", "Acrobatic", "Alt Hold ", "Auto     ", "Guided   ", "Loiter   ",
                     "RTL      ", "Circle   ", "Position ", "Land     ", "OF_Loiter", "Drift    ",
                     "None     ", "Sport    ", "Flip     ", "Auto Tune", "Pos Hold" };


ws2812 *driver;
uint32_t Mode1[18] = {COLOR(64,64,64), COLOR(64,0,0), COLOR(0,64,0), COLOR(0,64,0), COLOR(16,64,16),
                 COLOR(0,0,64), COLOR(64,0,0), COLOR(0,0,0), COLOR(0,0,0), COLOR(32,0,0),
                 COLOR(0,0,0), COLOR(16,0,16), COLOR(8,0,0), COLOR(8,0,0), COLOR(255,0,0),
                 COLOR(0,255,0), COLOR(0,0,64), COLOR(32,16,0)
               };
uint32_t Mode2[18] = {COLOR(64,64,64), COLOR(64,0,0), COLOR(0,64,0), COLOR(0,16,0), COLOR(0,64,0),
                 COLOR(0,0,64), COLOR(0,0,0), COLOR(0,0,0), COLOR(0,0,0), COLOR(0,32,0),
                 COLOR(0,0,0), COLOR(16,0,16), COLOR(0,0,8), COLOR(8,0,0), COLOR(255,0,0),
                 COLOR(0,0,128), COLOR(0,0,128), COLOR(0,16,32)
               };

uint32_t Leds[LCNT+1];               
fdserial *comm;
ws2812 *driver;
static volatile int CRat;
static volatile int Color;
static volatile int Modes;

mavdata_t Md;
mavlink_message_t *msg;
mavlink_message_t *buf;
mavlink_status_t *sts;
mavlink_heartbeat_t *Heart;
mavlink_sys_status_t *Status;
mavlink_gps_raw_int_t *GPS;
mavlink_vfr_hud_t *Hud;
mavlink_attitude_t *Att;
mavlink_mission_current_t *Msn;
mavlink_statustext_t *stx;

int chan = 0;
unsigned char Buffer[400];
unsigned char Out[100];
unsigned char Sts[400];
int Other = 0;


int main()
{
  int c;
  
  msg = (mavlink_message_t *)Buffer;
  sts = (mavlink_status_t *)Sts;
  buf = (mavlink_message_t *)Out;
  
  Md.Custom = 12;
  mavlink_system.sysid = 66;
  mavlink_system.compid = MAV_COMP_ID_IMU;
  CRat = 50;
  Leds[3] = COLOR(0,0,64);
  Leds[2] = COLOR(0,0,32);
  Leds[1] = COLOR(0,0,8);
  Leds[0] = COLOR(0,0,2);
  Color = 0;

  cog_run(&Reading, 40);
  pause(1000);
  comm = fdserial_open(Rx, Tx, FDSERIAL_MODE_NONE, Baud);

  mstime_start();
  cog_run(&Circle, 40);
  
  while(1)
  {
    c = fdserial_rxTime(comm, 500);
    if (c >= 0)
    {
	    if (mavlink_parse_char(chan, c, msg, sts))
      {
        if (MavDecode(msg) > 0)
            mstime_reset();
        memset(Buffer, 0, sizeof(Buffer));
        memset(Sts, 0, sizeof(Sts));
      }
      if (mstime_get() > 3000)
      {
        Md.Custom = 17;
        mstime_reset();
      }
    }
    else
    {
      if (mstime_get() > 5000)
      {
        Md.Custom = 7;
        mstime_reset();
      }
    }
    packet_drops += sts->packet_rx_drop_count;
    parse_errors += sts->parse_error;
  }
  fdserial_close(comm);  
}

int MavDecode(mavlink_message_t *m)
{
//  print("Message: %d, %d\n", m->msgid, m->seq);
  
  if (m->msgid == MAVLINK_MSG_ID_HEARTBEAT)
  {
    Heart = (mavlink_heartbeat_t *)m->payload64;
    if (Heart->base_mode & MAV_MODE_FLAG_DECODE_POSITION_SAFETY)
      Md.Armed = 1;
    else
      Md.Armed = 0;
    if (Heart->base_mode & MAV_MODE_FLAG_DECODE_POSITION_CUSTOM_MODE)
    {
      Md.Custom = Heart->custom_mode;
    }
    else
      Md.Custom = 0;
      
    Md.System = m->sysid;
    Md.Compent = m->compid;
    Md.Type = Heart->type;
    if (Other == 0)
    {
      request_mavlink_rates();
    }      
    return 2;
  }
  
  if (m->msgid != 253)
      Other = 1;
  
  if (m->msgid == MAVLINK_MSG_ID_SYS_STATUS)
  {
    Status = (mavlink_sys_status_t *)m->payload64;
    Md.BatteryRemaining = Status->battery_remaining;
    Md.BatteryVoltage = Status->voltage_battery;
    return 2;
  }
  
  if (m->msgid == MAVLINK_MSG_ID_GPS_RAW_INT)
  {
    GPS = (mavlink_gps_raw_int_t *)m->payload64;
    Md.Latitude = GPS->lat;
    Md.Longitude = GPS->lon;
    Md.Altitude = GPS->alt;
    Md.Satelites = GPS->satellites_visible;
    Md.HDOP = GPS->eph;
    Md.GPSFix = GPS->fix_type;
    return 1;
  }
  
  if (m->msgid == MAVLINK_MSG_ID_VFR_HUD)
  {
    Hud = (mavlink_vfr_hud_t *)m->payload64;
    Md.Heading = Hud->heading;
    Md.RelativeAltitude = Hud->alt;
    Md.Throttle = Hud->throttle;
    return 1;
  }
  
  if (m->msgid == MAVLINK_MSG_ID_ATTITUDE)
  {
    Att = (mavlink_attitude_t *)m->payload64;
    Md.Roll = Att->roll * 180/PI;
    Md.Pitch = Att->pitch * 180/PI;
    Md.Yaw = Att->yaw * 180/PI;
    return 1;
  }
  
  if (m->msgid == MAVLINK_MSG_ID_MISSION_CURRENT)
  {
    Msn = (mavlink_mission_current_t *)m->payload64;
    Md.WapPoint = Msn->seq;
    return 1;
  }
  
  if (m->msgid == MAVLINK_MSG_ID_STATUSTEXT)
  {
    stx = (mavlink_statustext_t *)m->payload64;
    stx->text[49] = 0;
//    print("status: %d->%s\n", stx->severity, stx->text);
  }    
  return 0;
}

void Reading()
{
  int i;
  
  pause(1000);
  
  while (1)
  {
    pause(500);
    Modes = Mode1[Md.Custom];
    if (Md.Roll > 3)
      FixColor(1);
    else
    {
      if (Md.Roll < -3)
        FixColor(2);
      else
        FixColor(0);
    }
    
    i = (Md.Throttle / 20) * 10;
    CRat = 50 - i;
//    ws2812_set(driver, LED_PIN, &Mode1[Md.Custom], 1);
    pause(500);
    Modes = Mode2[Md.Custom];
    if (Md.Roll > 5)
      FixColor(1);
    else
    {
      if (Md.Roll < -5)
        FixColor(2);
      else
        FixColor(0);
    }
    i = (Md.Throttle / 20) * 10;
    CRat = 50 - i;
//    ws2812_set(driver, LED_PIN, &Mode2[Md.Custom], 1);
  }
}

void Circle(void *par)
{
  int i;
  int j;

  driver = ws2812b_open();

  while(CRat)
  {
    for (i=1;i<17;i++)
    {
      j = i - 1;
      if (j < 1)
          j = 16;
     
      Leds[j] = Leds[i];
    }
    Leds[0] = Modes;
    ws2812_set(driver, LED_PIN, Leds, LCNT);

    pause(CRat);
  }
  ws2812_close(driver);
  cogstop(cogid());
}

void FixColor(int x)
{
  int j;
  
  for(int i=1;i<17;i++)
  {
    j = Leds[i];
    if (j > 0)
    {
      if (x > Color)
      {
        j = j << 8;
      }
      if (x < Color)
      {
        j = j >> 8;
      }
      if (x > Color+1)
      {
        j = j << 8;
      }
      if (x < Color-1)
      {
        j = j >> 8;
      }
    }
    Leds[i] = j;
  }
  Color = x;
}

void request_mavlink_rates()
{
  int i,j;
  int t;
  
  const int  maxStreams = 1;
  const uint8_t MAVStreams[] = {
        MAV_DATA_STREAM_ALL,
        MAV_DATA_STREAM_RAW_SENSORS,
        MAV_DATA_STREAM_EXTENDED_STATUS,
        MAV_DATA_STREAM_RC_CHANNELS,
        MAV_DATA_STREAM_POSITION,
        MAV_DATA_STREAM_EXTRA1, 
        MAV_DATA_STREAM_EXTRA2};
  const uint16_t MAVRates[] = {0x01, 0x02, 0x05, 0x02, 0x05, 0x02, 0x02};
  for (i=0; i < maxStreams; i++)
  {
    t = mavlink_msg_request_data_stream_pack(mavlink_system.sysid, mavlink_system.compid, buf,
          Md.System, Md.Compent, MAVStreams[i], MAVRates[i], 1);
    for (j=2;j<t;j++)
    {
      fdserial_txChar(comm, Out[j]);
//      print("%2x ", Out[j]);
    }
    fdserial_txChar(comm, Out[0]);
//    print("%2x ", Out[j]);
    fdserial_txChar(comm, Out[1]);
//    print("%2x \n", Out[j]);
  }
//    print("setup: %d\n", buf->seq);
}

